import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException
import com.urbancode.air.AirPluginTool
import com.urbancode.shell.Shell;

final airTool = new AirPluginTool(args[0], args[1])
final Properties props = airTool.stepProperties
final workDir = new File('.').canonicalFile
final String osName = System.getProperty('os.name').toLowerCase(Locale.US)
final boolean windows = (osName =~ /windows/)
final File PLUGIN_HOME = new File(System.getenv().get("PLUGIN_HOME"))



String version = props['version'];
int retry = Integer.valueOf(props['retry']);
println "Retries : " + String.valueOf(retry);
int interval = Integer.valueOf(props['interval']);
println "Interval : " + String.valueOf(interval);
int timeout = Integer.valueOf(props['timeout']);
println "Timeout : " + String.valueOf(timeout);
String namingPort = props['namingPort'];
println "Port : " + namingPort;
String hostName = props['hostname'] ? props['hostname'] : 'localhost';
println "HostName : " + hostName;
String jbossLauncher = props['jbossLauncher'];
println "Launcher : " + jbossLauncher;
String options = props['options'];
println "Options : " + options?.split('[\r\n]')?.join(' ');
String username = props['username'];
String password = props['password'];
println '--------------------------------------'

def String getArch() {
    String result
    String arch = System.getProperty("os.arch").toLowerCase(Locale.US)

    if (arch.indexOf("amd64") > -1 || arch.indexOf("x64") > -1 || arch.indexOf("x86_64") > -1) {
        result = "x64"
    }
    else if (arch.indexOf("x86") > -1 || arch.indexOf("386") > -1 || arch.indexOf("486") > -1 ||
            arch.indexOf("586") > -1 || arch.indexOf("686") > -1 || arch.indexOf("pentium") > -1) {
        result = "x86"
    }
    else if (arch.indexOf("ia64") > -1 || arch.indexOf("itanium") > -1 || arch.indexOf("ia-64") > -1) {
        result = "ia64"
    }
    else if (arch.indexOf("ppc") > -1 || arch.indexOf("powerpc") > -1) {
        result = "ppc"
    }
    else if (arch.indexOf("sparc") > -1) {
        result = "sparc"
    }
    else if (arch.indexOf("parisc") > -1 || arch.indexOf("pa_risc") > -1 || arch.indexOf("pa-risc") > -1) {
        result = "parisc"
    }
    else if (arch.indexOf("alpha") > -1) {
        result = "alpha"
    }
    else if (arch.indexOf("mips") > -1) {
        result = "mips"
    }
    else if (arch.indexOf("arm") > -1) {
        result = "arm"
    }
    else {
        result = "unknown"
    }
    return result
}

def startJBoss = { commandArray ->
    // we need to use Shell with daemon option because of Windows and maybe some custom *nix startup scripts that
    // do not close file handles.
    def Shell shell = new Shell(commandArray as String[])
    shell.workingDirectory = workDir
    shell.daemon = true
    shell.addEnvironmentVariable('LAUNCH_JBOSS_IN_BACKGROUND', '1')
    shell.execute()
}

def connectToJboss = { retryCount ->
    InitialContext ctx = null;
    try {
        if (version == "5.x") {
            Properties p = new Properties();
            p.put("java.naming.factory.initial",
                    "org.jnp.interfaces.NamingContextFactory");
            p.put("java.naming.factory.url.pkgs",
                    "org.jboss.naming:org.jnp.interfaces");
            p.put("java.naming.provider.url", hostName + ":" + namingPort);
            p.put("jnp.disableDiscovery", "true");
            ctx = new InitialContext(p);
            Class SecurityClient = Class.forName("org.jboss.security.client.SecurityClient");
            Class SecurityClientFactory = Class.forName("org.jboss.security.client.SecurityClientFactory");
            def client = SecurityClientFactory.getSecurityClient();
            client.setSimple(username, password);
            client.login();
        } else {
            Properties p = new Properties();
            p.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
            p.setProperty(Context.URL_PKG_PREFIXES, "org.jboss.naming:org.jnp.interfaces");
            p.setProperty(Context.PROVIDER_URL, hostName + ":" + namingPort);
            p.setProperty(Context.SECURITY_PRINCIPAL, username);
            p.setProperty(Context.SECURITY_CREDENTIALS, password);
            p.setProperty("jnp.disableDiscovery", "true");
            ctx = new InitialContext(p);
        }

    }
    catch (NamingException e) {
        throw new RuntimeException("Unable to instantiate naming context: " + e.getMessage(), e)
    }

    // Try to get JBoss jmx MBean connection
    MBeanServerConnection s = null;
    int i = 0;
    while (i < retryCount) {
        println("Attempt to retrieve JBoss JMX MBean connection");
        try {
            s = (MBeanServerConnection) ctx.lookup("jmx/invoker/RMIAdaptor");
        }
        catch (NamingException e) {}
        if (s == null) {
            i++
        }
        else {
            break
        }
        Thread.sleep(interval)
    }
    return s
}

if (windows) {
    def arch = getArch()
    def libraryPath = new File(PLUGIN_HOME, "lib/native/${arch}/WinAPI.dll")
    System.setProperty("com.urbancode.winapi.WinAPI.dllPath", libraryPath.absolutePath)
}

//start jboss using launcher
def cmdArgs = [jbossLauncher]
options.split('[\r\n]').each {
    if (it && it.trim().length() > 0) {
        cmdArgs << it.trim();
    }
}

try {
    println "Check if JBoss is already running..."
    MBeanServerConnection s = null
    try {
        s = connectToJboss(1)
    }
    catch (Exception e) {}

    if (s) {
        println "JBoss is already running, skip launcher execution."
    }
    else {
        println "JBoss is not running, executing:\n\t" + cmdArgs.join(' ')
        startJBoss cmdArgs
    }

    s = connectToJboss(retry)
    
    if (!s) {
        throw new RuntimeException("Unable to get JBoss jmx MBean connection.")
    }

    ObjectName infoMBean = new ObjectName("jboss.system:type=ServerInfo");
    println "Connected to: " + s.getAttribute(infoMBean, "HostAddress");
    ObjectName serverConfigMBean = new ObjectName("jboss.system:type=ServerConfig");
    println "Configuration Name: " + s.getAttribute(serverConfigMBean, "ServerName");
    println 'Waiting for server to change to Started state...'

    // Wait server is started
    boolean isStarted = false;
    long startTime = System.currentTimeMillis();
    while (!isStarted && System.currentTimeMillis() - startTime < timeout) {
        try {
            ObjectName serverMBeanName = new ObjectName("jboss.system:type=Server");
            isStarted = ((Boolean) s.getAttribute(serverMBeanName, "Started")).booleanValue();
            if (!isStarted) {
                Thread.sleep(interval)
            }
            else {
                println "Server started."
            }
        }
        catch (Exception e) {
            throw new RuntimeException("Unable to wait: " + e.getMessage(), e);
        }
    }
    if (!isStarted) {
        throw new RuntimeException(
                "JBoss AS is not stared before timeout has expired! ");
    }
}
catch (Exception e) {
    e.printStackTrace();
    System.exit(1);
}
