import javax.management.MBeanServerConnection;
import javax.management.ObjectName;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;

Properties props = new Properties();
File propsFile = new File(this.args[0]);
InputStream stream;
try {
    stream = new FileInputStream(propsFile);
    props.load(stream);
}
catch (Exception e) {
    e.printStackTrace();
    System.exit(1);
}
finally {
    if (stream != null) {
        stream.close();
    }
}

String version = props['version'];
int retry = Integer.valueOf(props['retry']);
println "Retries : " + String.valueOf(retry);
int interval = Integer.valueOf(props['interval']);
println "Interval : " + String.valueOf(interval);
int timeout = Integer.valueOf(props['timeout']);
println "Timeout : " + String.valueOf(timeout);
String namingPort = props['namingPort'];
println "Port : " + namingPort;
String hostName = props['hostname']?props['hostname']:'localhost';
println "HostName : " + hostName;
String jbossLauncher = props['jbossLauncher'];
println "Launcher : " + jbossLauncher;
String options = props['options'];
println "Options : " + options;
String username = props['username'];
String password = props['password'];


//start jboss using launcher

def cmdArgs = [jbossLauncher];
for (String option : options.split('\n')) {
    cmdArgs << option.trim();
}

try {
    println "Executing " + cmdArgs.join(' ');
    def proc = cmdArgs.execute();

    InitialContext ctx = null;
    try {
      if (version != "other") {
          println "Logging in...";
          Properties p = new Properties();
          p.put("java.naming.factory.initial",
              "org.jnp.interfaces.NamingContextFactory");
          p.put("java.naming.factory.url.pkgs",
              "org.jboss.naming:org.jnp.interfaces");
          p.put("java.naming.provider.url", hostName+":"+namingPort);
          p.put("jnp.disableDiscovery", "true");
          ctx = new InitialContext(p);
          Class SecurityClient = Class.forName("org.jboss.security.client.SecurityClient");
          Class SecurityClientFactory = Class.forName("org.jboss.security.client.SecurityClientFactory");
          def client = SecurityClientFactory.getSecurityClient();
          client.setSimple(username, password);
          client.login();
      }
      else {
          Properties p = new Properties();
          p.setProperty(Context.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
          p.setProperty(Context.URL_PKG_PREFIXES, "org.jboss.naming:org.jnp.interfaces");
          p.setProperty(Context.PROVIDER_URL, hostName+":"+namingPort);
          p.setProperty(Context.SECURITY_PRINCIPAL, username);
          p.setProperty(Context.SECURITY_CREDENTIALS, password);
          p.setProperty("jnp.disableDiscovery", "true");
          ctx = new InitialContext(p);
      }

    }
    catch (NamingException e) {
        throw new RuntimeException("Unable to instantiate naming context: " + e.getMessage(), e)
    }

    // Try to get JBoss jmx MBean connection
    MBeanServerConnection s = null;
    int i = 0;
    while (i < retry) {
      try {
        println "Trying to get connection again!";
        s = (MBeanServerConnection) ctx.lookup("jmx/invoker/RMIAdaptor");

        if (s == null) {
           //cannot connect it seems
           System.out.println("Could not connect to jboss! Must now be down!");
           System.exit(0);
        }

        i++;
        Thread.sleep(interval);
      } catch (NamingException e) {
        //cannot connect it worked
        System.out.println("Could not connect to jboss! Must be down!");
        System.exit(0);
      }
    }
    throw new RuntimeException("Could not shutdown jboss!");
}
catch (Exception e) {
    e.printStackTrace();
    System.exit(1);
}
